<?php

namespace Bydedi\Security\EventListener;

use Bydedi\Http\Event\MatchRequestEvent;
use Bydedi\Http\HttpEvents;
use Bydedi\Security\Event\PermissionDeniedEvent;
use Bydedi\Security\Event\RoleDeniedEvent;
use Bydedi\Security\Event\SecurityEvent;
use Bydedi\Security\Locker;
use Bydedi\Security\Role;
use Bydedi\Security\SecurityEvents;
use Exception;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Psr\Log\LoggerInterface;
use Bydedi\Security\Permission;
use Symfony\Component\HttpFoundation\Response;

/**
 * class LockerListener
 */
class LockerListener implements EventSubscriberInterface
{
    /**
     * @var \Bydedi\Security\Locker
     */
    private $locker;

    /**
     * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
     */
    protected $dispatcher;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * Constructor.
     *
     * @param Locker $locker The Url or Request matcher
     * @param EventDispatcherInterface $dispatcher
     * @param LoggerInterface $logger The logger
     *
     */
    public function __construct(Locker $locker, EventDispatcherInterface $dispatcher, LoggerInterface $logger = null)
    {
        $this->locker = $locker;
        $this->dispatcher = $dispatcher;
        $this->logger = $logger;
    }

    /**
     * @param MatchRequestEvent $event
     * @throws Exception
     */
    public function onRequestRole(MatchRequestEvent $event)
    {
        $request = $event->getRequest();

        $roleName = $request->attributes->get('role');
        if (!empty($roleName)) {
            $roles = $request->getSession()->get('roles');

            if (empty($roles) || in_array($roleName, $roles) == false) {

                $role = $this->locker->findRoleByName($roleName);

                if (!$role instanceof Role) {
                    throw new Exception(sprintf('role name "%s" doesn\'t exist', $roleName));
                }

                $eventRDE = new RoleDeniedEvent($roles, $role, $request);
                $this->dispatcher->dispatch(SecurityEvents::ROLE_DENIED, $eventRDE);

                if ($eventRDE->hasResponse()) {
                    $event->setResponse($eventRDE->getResponse());
                } else {
                    // default response if no response in event
                    $event->setResponse(new Response('Access denied'));
                }

            } else {
                $this->dispatcher->dispatch(SecurityEvents::ACCESS, new SecurityEvent($roles, $request));
            }
        }
    }

    /**
     * @param MatchRequestEvent $event
     * @throws Exception
     */
    public function onRequestPermission(MatchRequestEvent $event)
    {
        $request = $event->getRequest();

        $permissionCode = $request->attributes->get('permission');
        if (!empty($permissionCode)) {
            $permission = $this->locker->findPermissionByCode($permissionCode);

            if (!$permission instanceof Permission) {
                throw new Exception('permission code ' . $permissionCode . ' doesn\'t exist');
            }

            $roles = $request->getSession()->get('roles');

            if ($this->locker->can($roles, $permission) == false) {

                if (null !== $this->logger) {
                    // TODO : example below
                    // $this->logger->info(sprintf('Access denied route "%s" (parameters: %s)', $parameters['_route'], $this->parametersToString($parameters)));
                }

                $eventPDE = new PermissionDeniedEvent($roles, $permission, $request);
                $this->dispatcher->dispatch(SecurityEvents::PERMISSION_DENIED, $eventPDE);

                if ($eventPDE->hasResponse()) {
                    $event->setResponse($eventPDE->getResponse());
                } else {
                    // default response if no response in event
                    $event->setResponse(new Response('Access denied'));
                }

                // throw new PermissionDeniedException('permission denied : you can\'t access to this ressource');

            } else {

                if (null !== $this->logger) {
                    // TODO : example below
                    // $this->logger->info(sprintf('Access route "%s" (parameters: %s)', $parameters['_route'], $this->parametersToString($parameters)));
                }

                $this->dispatcher->dispatch(SecurityEvents::ACCESS, new SecurityEvent($roles, $request));
            }
        }
    }

    public static function getSubscribedEvents()
    {
        return array(
            HttpEvents::REQUEST => [
                ['onRequestRole'],
                ['onRequestPermission']
            ],
        );
    }
}
